// @flow

import React from 'react'
import { isEmpty } from 'lodash'
import styled from 'styled-components/native'
import { Form, Field } from 'react-final-form'

type Props = {
  styles?: { outerStyles?: string, innerStyles?: string },
  onSubmit: Function,
  Button?: { component: Function, options?: Object },
}

const Container = styled.View`
  ${({ userStyles }) => userStyles}
`
const InnerContainer = styled.View`
  border-radius: 4;
  ${({ userStyles }) => userStyles}
`

const FieldContainer = styled.View`
  ${({ userStyles }) => userStyles}
`

const ErrorContainer = styled.View`
  justify-content: center;
  align-items: center;
  height: 26;
  ${({ userStyles }) => userStyles}
`

const ErrorText = styled.View`
  ${({ userStyles }) => userStyles}
`

export default ({ onSubmit, validator, styles = {}, ...props }: Props) => {
  if (!onSubmit && typeof onSubmit !== 'function') {
    throw new Error('Your FormBuilder needs an onSubmit prop')
  }

  if (isEmpty(props)) {
    throw new Error('Your FormBuilder needs some form field, pass them in as props of any capitalized name.')
  }

  /*  Makes sure these props aren't rendered as fields
      All other props will be assumed to be fields.
  */
  const validFields = Object.keys(props).filter(
    p => !['options', 'onSubmit', 'Button', 'styles', 'initialValues', 'validator'].includes(p),
  )

  const [outerContainer, errorContainer, innerContainer, fieldContainer, errorText] = [
    styles.outerContainer || {},
    styles.errorContainer || {},
    styles.innerContainer || {},
    styles.fieldContainer || {},
    styles.errorText || {},
  ]

  return (
    <Container userStyles={outerContainer}>
      <Form
        onSubmit={onSubmit}
        validate={validator}
        initialValues={props.initialValues || {}}
        fields={validFields}
        render={({ handleSubmit, errors, dirty, submitFailed }) => {
          return (
            <>
              <InnerContainer userStyles={innerContainer}>
                {validFields.map(propName => {
                  const p = props[propName]
                  const { Input, options } =
                    typeof p === 'object'
                      ? { Input: p.render || p.component, options: p.options || {} }
                      : { Input: p, options: {} }

                  return (
                    <FieldContainer key={propName} userStyles={fieldContainer}>
                      <Field
                        name={propName.toLowerCase()}
                        render={({ placeholder, input, meta }) => {
                          return (
                            <Input
                              placeholder={placeholder}
                              {...options}
                              input={{ ...input, ...options.input }}
                              meta={meta}
                            />
                          )
                        }}
                      />
                    </FieldContainer>
                  )
                })}
              </InnerContainer>
              {submitFailed ? (
                <ErrorContainer userStyles={errorContainer}>
                  {Object.keys(errors).map(err => (
                    <ErrorText userStyles={errorText}>{errors[err]}</ErrorText>
                  ))}
                </ErrorContainer>
              ) : (
                <ErrorContainer userStyles={errorContainer} />
              )}
              {!!props.Button && (
                <props.Button.component type="submit" onPress={handleSubmit}>
                  <props.Button.options.render />
                </props.Button.component>
              )}
            </>
          )
        }}
      />
    </Container>
  )
}
